==> JSL-MT32_04.zip: RolandMT32SingleTimbreDriver.java <==
/*
 * Copyright 2004 Fred Jan Kraan
 *
 * This file is part of JSynthLib.
 *
 * JSynthLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * JSynthLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSynthLib; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*
 * RolandMT32SingleTimbreDriver.java v.0.4
 * part of the Roland MT-32 driver
 */

package synthdrivers.RolandMT32;
import core.Driver;
import core.JSLFrame;
import core.NameValue;
import core.Patch;
import core.SysexHandler;
import core.ErrorMsg;

/**
 * Single Voice Patch Driver for Roland MT32.
 *
 * @version $Id: RolandMT32SingleDriver.java,v 0.02 2004/07/10 21:11:39 fjkraan Exp $
 */
public class RolandMT32SingleTimbreDriver extends Driver {
    /** Header Size of the Data set DT1 message. */
    private static final int HSIZE = 8;
    /** Single Patch size */
    private static final int SSIZE = 247;
    String TTABaseAddress="04";
    String TTAPartAddress="00 00";
    private static final SysexHandler SYS_REQ = new SysexHandler(
        "F0 41 10 16 11 04 *partAddrM* *partAddrL* 00 01 76 *checkSum* F7");

    public RolandMT32SingleTimbreDriver() {
	super("Single Timbre", "Fred Jan Kraan");
	sysexID = "F041**16";
	patchSize	= HSIZE + SSIZE + 1;
	patchNameStart	= HSIZE;
	patchNameSize	= 10;
	deviceIDoffset	= 2;
        checksumStart=5;
        checksumEnd=10;
        checksumOffset=0;
//	checksumStart	= HSIZE;
//	checksumEnd	= HSIZE + SSIZE - 2;
//	checksumOffset	= HSIZE + SSIZE - 1;
        bankNumbers =new String[] {"0-Internal"};
        patchNumbers=new String[] {"TTA-1","TTA-2","TTA-3","TTA-4","TTA-5","TTA-6","TTA-7","TTA-8"};  
    }

    /* Send a patch (bulk dump system exclusive message) to MIDI device. 
     * Target should be Timbre Memory 1 - 64. TTA will do for now. 
     * The message format here is Data set DT1 */
    public void storePatch(Patch p, int bankNum, int patchNum) {
	setBankNum(bankNum);    // Control change
	setPatchNum(patchNum);  // Program change
	try {
	    Thread.sleep(100);
	} catch (Exception e) {
            ErrorMsg.reportStatus(e);
	}

//	int timbreAddr = patchNum * (SSIZE - 1);
	int timbreAddr = patchNum * 0x100;
	int timAddrM = timbreAddr / 0x80;
	int timAddrL = timbreAddr & 0x7F;	

        p.sysex[0]   = (byte) 0xF0;
//	p.sysex[2]   = (byte) 0x10;
        p.sysex[5]   = (byte) 0x04;     // point to Timbre Memory
	p.sysex[6]   = (byte) timAddrM;
	p.sysex[7]   = (byte) timAddrL;

        System.out.println ("store patchNum " + patchNum + " to timAddrM/L " + timAddrM + " / " + timAddrL);
//	sendPatchWorker(p);
//      send(p.sysex);
	try {
            sendPatchWorker(p);
	    Thread.sleep(100);
	} catch (Exception e) {
            ErrorMsg.reportStatus(e);
	}
	setPatchNum(patchNum);  // Program change
    }

    /*  Send a Patch (bulk dump system exclusive message) to an edit
     *  buffer of MIDI device. Target should be Timbre Temp Area 1 - 8. 
     *  The message format here is Data set DT1 */
    public void sendPatch(Patch p) {
//        int patchNum = (HSIZE + SSIZE + 1);
//	p.sysex[0] = (byte) 0xF0;
//	p.sysex[1] = (byte) 0x41;
//	p.sysex[2] = (byte) 0x10;
//	p.sysex[3] = (byte) 0x16;
// 	p.sysex[4] = (byte) 0x08;
//	p.sysex[5] = (byte) 0x00;
//	p.sysex[6] = (byte) 0x00;
// 	p.sysex[4] = (byte) 0x04;
//	p.sysex[5] = (byte) (byte)(patchNum / 0x80);
//	p.sysex[6] = (byte) (byte)(patchNum & 0x7F);

        //this.calculateChecksum(p, 4, 253, 0);
// 	p.sysex[255] = (byte) 0xF7;

//        System.out.println("sendPatch: Not implemented yet.");
	sendPatchWorker(p);
    }

    // not used
    public void calculateChecksum(Patch p, int start, int end, int ofs) {
	int sum = 0;
	for (int i = start; i <= end; i++) {
	    sum += p.sysex[i];
	}
	sum += 0xA5;
	p.sysex[ofs] = (byte) (sum % 128);
    }

    // not used
    public Patch createNewPatch() {
        byte[] sysex = new byte[HSIZE + SSIZE + 1];
	sysex[0] = (byte) 0xF0; sysex[1] = (byte) 0x41; sysex[2] = (byte) 0x10;
	sysex[3] = (byte) 0x16; sysex[4] = (byte) 0x12; sysex[5] = (byte) 0x04;
	sysex[6] = (byte) 0x0; sysex[HSIZE + SSIZE] = (byte) 0xF7;
	Patch p = new Patch(sysex, this);
	setPatchName(p, "New Timbre");
	calculateChecksum(p);
	return p;
    }

    public JSLFrame editPatch(Patch p) {
	return new RolandMT32SingleTimbreEditor(p);
    }

    public void requestPatchDump(int bankNum, int timNum) {
      // timNum misuses patchNum
        // The message format here is Request RQ1
        NameValue nv[]=new NameValue[3];
        int timbreAddr = timNum * 0xF6;
        int timSizeH = 0x00;
        int timSizeM = 0x01;
        int timSizeL = 0x76;
        int timAddrH = 0x04;               // Always Timbre Temp Area
        int timAddrM = timbreAddr / 0x80;  // timbreAddr >> 8
        int timAddrL = timbreAddr & 0x7F;
        nv[0]=new NameValue("partAddrM", timAddrM);
        nv[1]=new NameValue("partAddrL", timAddrL);
        int checkSum = ( 0 - (timAddrH + timAddrM + timAddrL + timSizeH + timSizeM + timSizeL)) & 0x7F;
        nv[2]=new NameValue("checkSum",checkSum);
//        nv[3]=new NameValue("bankNum", bankNum << 1);
//        nv[4]=new NameValue("timNum", timNum );
        
        send(SYS_REQ.toSysexMessage(getChannel(),  nv));
    }
}

==> JSL-MT32_05.zip: RolandMT32SingleTimbreDriver.java <==
/*
 * Copyright 2004 Fred Jan Kraan
 *
 * This file is part of JSynthLib.
 *
 * JSynthLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * JSynthLib is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JSynthLib; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/**
 * Single Timbre Temp Driver for Roland MT32.
 *
 * @version $Id: RolandMT32SingleDriver.java,v 0.05 2004/07/24 16:46:00 fjkraan Exp $
 */

package synthdrivers.RolandMT32;
import core.Driver;
import core.JSLFrame;
import core.NameValue;
import core.Patch;
import core.SysexHandler;
import core.ErrorMsg;


public class RolandMT32SingleTimbreDriver extends Driver {
    /** Header Size of the Data set DT1 message. */
    private static final int HSIZE = 8;
    /** Single Patch size */
    private static final int SSIZE = 247;
    private static final SysexHandler SYS_REQ = new SysexHandler(
        "F0 41 10 16 11 04 *partAddrM* *partAddrL* 00 01 76 *checkSum* F7");

    public RolandMT32SingleTimbreDriver() {
	super("Single Timbre", "Fred Jan Kraan");
	sysexID = "F041**16";
	patchSize	= HSIZE + SSIZE + 1;
	patchNameStart	= HSIZE;
	patchNameSize	= 10;
	deviceIDoffset	= 2;
        checksumStart=5;
        checksumEnd=10;
        checksumOffset=0;
        bankNumbers =new String[] {"0-Internal"};
        patchNumbers=new String[] {"TTA-1","TTA-2","TTA-3","TTA-4","TTA-5","TTA-6","TTA-7","TTA-8"};  
    }

    /* Send a patch (bulk dump system exclusive message) to MIDI device. 
     * Target should be Timbre Memory 1 - 64. TTA will do for now. 
     * The message format here is Data set DT1 */
    public void storePatch(Patch p, int bankNum, int patchNum) {
	setBankNum(bankNum);    // Control change
	setPatchNum(patchNum);  // Program change
	try {
	    Thread.sleep(100);
	} catch (Exception e) {
            ErrorMsg.reportStatus(e);
	}

//	int timbreAddr = patchNum * (SSIZE - 1);
	int timbreAddr = patchNum * 0x100;
	int timAddrM = timbreAddr / 0x80;
	int timAddrL = timbreAddr & 0x7F;	

        p.sysex[0]   = (byte) 0xF0;
//	p.sysex[2]   = (byte) 0x10;
        p.sysex[5]   = (byte) 0x04;     // point to Timbre Memory
	p.sysex[6]   = (byte) timAddrM;
	p.sysex[7]   = (byte) timAddrL;

        System.out.println ("store patchNum " + patchNum + " to timAddrM/L " + timAddrM + " / " + timAddrL);
//	sendPatchWorker(p);
//      send(p.sysex);
	try {
            sendPatchWorker(p);
	    Thread.sleep(100);
	} catch (Exception e) {
            ErrorMsg.reportStatus(e);
	}
	setPatchNum(patchNum);  // Program change
    }

    /*  Send a Patch (bulk dump system exclusive message) to an edit
     *  buffer of MIDI device. Target should be Timbre Temp Area 1 - 8. 
     *  The message format here is Data set DT1 */
    public void sendPatch(Patch p) {
//        int patchNum = (HSIZE + SSIZE + 1);
//	p.sysex[0] = (byte) 0xF0;
//	p.sysex[1] = (byte) 0x41;
//	p.sysex[2] = (byte) 0x10;
//	p.sysex[3] = (byte) 0x16;
// 	p.sysex[4] = (byte) 0x08;
//	p.sysex[5] = (byte) 0x00;
//	p.sysex[6] = (byte) 0x00;
// 	p.sysex[4] = (byte) 0x04;
//	p.sysex[5] = (byte) (byte)(patchNum / 0x80);
//	p.sysex[6] = (byte) (byte)(patchNum & 0x7F);

        //this.calculateChecksum(p, 4, 253, 0);
// 	p.sysex[255] = (byte) 0xF7;

//        System.out.println("sendPatch: Not implemented yet.");
	sendPatchWorker(p);
    }

    // not used
    public void calculateChecksum(Patch p, int start, int end, int ofs) {
	int sum = 0;
	for (int i = start; i <= end; i++) {
	    sum += p.sysex[i];
	}
	sum += 0xA5;
	p.sysex[ofs] = (byte) (sum % 128);
    }

    // not used
    public Patch createNewPatch() {
        byte[] sysex = new byte[HSIZE + SSIZE + 1];
	sysex[0] = (byte) 0xF0; sysex[1] = (byte) 0x41; sysex[2] = (byte) 0x10;
	sysex[3] = (byte) 0x16; sysex[4] = (byte) 0x12; sysex[5] = (byte) 0x04;
	sysex[6] = (byte) 0x0; sysex[HSIZE + SSIZE] = (byte) 0xF7;
	Patch p = new Patch(sysex, this);
	setPatchName(p, "New Timbre");
	calculateChecksum(p);
	return p;
    }

    public JSLFrame editPatch(Patch p) {
	return new RolandMT32SingleTimbreEditor(p);
    }

    public void requestPatchDump(int bankNum, int timNum) {
      // timNum misuses patchNum
        // The message format here is Request RQ1
        NameValue nv[]=new NameValue[3];
        int timbreAddr = timNum * (SSIZE -1); //0xF6;
        int timSizeH = 0x00;
        int timSizeM = 0x01;
        int timSizeL = 0x76;
        int timAddrH = 0x04;               // Always Timbre Temp Area
        int timAddrM = timbreAddr / 0x80;  // timbreAddr >> 8
        int timAddrL = timbreAddr & 0x7F;
        nv[0]=new NameValue("partAddrM", timAddrM);
        nv[1]=new NameValue("partAddrL", timAddrL);
        int checkSum = ( 0 - (timAddrH + timAddrM + timAddrL + timSizeH + timSizeM + timSizeL)) & 0x7F;
        nv[2]=new NameValue("checkSum",checkSum);
        
        send(SYS_REQ.toSysexMessage(getChannel(),  nv));
    }
}
